# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from silva.core.interfaces import IVersion,  IVersionedContent
from silva.core.contentlayout.interfaces import IPage as IBasicPage
from silva.core.contentlayout.interfaces import IPageAware


class IPageContent(IVersionedContent, IPageAware):
    """ Basic page feature

    This content supports advanced inline editing and content
    layout. Use this class to extend a page.
    """


class IPage(IPageContent):
    """A page
    """


class IPageContentVersion(IVersion, IBasicPage):
   """Basic page version feature
   """


class IPageVersion(IPageContentVersion):
    """A page version
    """




# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest
from datetime import datetime

from Products.Silva.testing import TestRequest, Transaction
from Products.Silva.tests.test_xml_export import SilvaXMLTestCase

from silva.core.contentlayout.interfaces import IBlockController, IBlockManager
from silva.core.contentlayout.blocks.slot import BlockSlot
from silva.core.contentlayout.blocks.text import TextBlock
from silva.core.contentlayout.designs.registry import registry
from silva.core.interfaces import IPublicationWorkflow
from silva.app.news.interfaces import IServiceNews
from zeam.component import getWrapper
from zope.component import getUtility

from ..news.blocks import NewsInfoBlock, AgendaInfoBlock
from ..testing import FunctionalLayer


class PageXMLExportTestCase(SilvaXMLTestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        with Transaction():
            self.layer.login('editor')
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addFolder('base', 'Export base')
            factory = self.root.base.manage_addProduct['silva.app.page']
            factory.manage_addPage('page', 'Page')
            design = registry.lookup_design_by_name('adesign')
            version = self.root.base.page.get_editable()
            version.set_design(design)

    def test_export_page(self):
        """Export a simple page.
        """
        exporter = self.assertExportEqual(
            self.root.base,
            'test_export_page.silvaxml')
        self.assertEqual(exporter.getZexpPaths(), [])
        self.assertEqual(exporter.getAssetPaths(), [])
        self.assertEqual(exporter.getProblems(), [])

    def test_export_page_with_page_model(self):
        """Export a page with a page model at the same time.
        """
        with Transaction():
            factory = self.root.base.manage_addProduct['silva.core.contentlayout']
            factory.manage_addPageModel('model', 'A Page Model')

            model = self.root.base.model
            model_version = model.get_editable()
            model_version.set_design(registry.lookup_design_by_name('adesign'))
            IPublicationWorkflow(model).publish()

            text_block = TextBlock(identifier='text block 1')
            controller = getWrapper(
                (text_block, model_version, TestRequest()),
                IBlockController)
            controller.text = "<div>text</div>"

            manager = IBlockManager(model_version)
            manager.add('two', text_block)
            manager.add('two', BlockSlot(identifier='slot-for-two'))
            manager.add('one', BlockSlot(
                    identifier='slot-for-one',
                    tag='div',
                    css_class='the-only-one'))

            page_version = self.root.base.page.get_editable()
            page_version.set_design(model_version)

        self.assertEquals(model_version, page_version.get_design())

        exporter = self.assertExportEqual(
            self.root.base,
            'test_export_page_with_page_model.silvaxml')
        self.assertEqual(exporter.getZexpPaths(), [])
        self.assertEqual(exporter.getAssetPaths(), [])
        self.assertEqual(exporter.getProblems(), [])

    def test_export_page_with_external_page_model(self):
        """Export a page that uses a page model that is not inside the
        export folder.
        """
        with Transaction():
            factory = self.root.manage_addProduct['silva.core.contentlayout']
            factory.manage_addPageModel('model', 'A Page Model')
            IPublicationWorkflow(self.root.model).publish()

            model_version = self.root.model.get_viewable()
            page_version = self.root.base.page.get_editable()
            page_version.set_design(model_version)

        self.assertEquals(model_version, page_version.get_design())

        exporter = self.assertExportEqual(
            self.root.base,
            'test_export_page_with_external_page_model.silvaxml')
        self.assertEqual(exporter.getZexpPaths(), [])
        self.assertEqual(exporter.getAssetPaths(), [])
        self.assertEqual(exporter.getProblems(), [])


class NewsPageXMLExportTestCase(SilvaXMLTestCase):
    layer = FunctionalLayer

    def setUp(self):
        # adding base folder
        self.root = self.layer.get_application()
        with Transaction():
            self.layer.login('editor')
            factory = self.root.manage_addProduct['silva.app.news']
            factory.manage_addNewsPublication('news', 'News')
            self.root.news.filter.set_show_agenda_items(True)
            service = getUtility(IServiceNews)
            service.add_subject('all', 'All')
            service.add_subject('other', 'Others')
            service.add_target_audience('generic', 'Generic')

    def test_export_news_info_page(self):
        # adding news page
        with Transaction():
            factory = self.root.news.manage_addProduct['silva.app.page']
            factory.manage_addNewsPage('newspage', 'A News Page')

        version = self.root.news.newspage.get_editable()
        design = registry.lookup_design_by_name('adesign')
        self.assertIsNotNone(design)
        self.assertIsNotNone(version)

        with Transaction():
            version.set_design(design)
            version.set_subjects(['all'])
            version.set_target_audiences(['generic'])
            version.set_display_datetime(datetime(2010, 9, 30, 10, 0, 0))

        # adding blocks
        IBlockManager(version).add('one', NewsInfoBlock())

        exporter = self.assertExportEqual(
            self.root.news,
            'test_export_news_page.silvaxml')
        self.assertEqual(exporter.getZexpPaths(), [])
        self.assertEqual(exporter.getAssetPaths(), [])
        self.assertEqual(exporter.getProblems(), [])

    def test_export_agenda_info_page(self):
        # adding agenda page
        with Transaction():
            factory = self.root.news.manage_addProduct['silva.app.page']
            factory.manage_addAgendaPage('agendapage', 'An Agenda Page')
            version = self.root.news.agendapage.get_editable()
            design = registry.lookup_design_by_name('adesign')

        self.assertIsNotNone(version)
        self.assertIsNotNone(design)

        with Transaction():
            version.set_design(design)
            # adding block
            IBlockManager(version).add('one', AgendaInfoBlock())

        exporter = self.assertExportEqual(
            self.root.news,
            'test_export_agenda_page.silvaxml')
        self.assertEqual(exporter.getZexpPaths(), [])
        self.assertEqual(exporter.getAssetPaths(), [])
        self.assertEqual(exporter.getProblems(), [])


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(PageXMLExportTestCase))
    suite.addTest(unittest.makeSuite(NewsPageXMLExportTestCase))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from five import grok

from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass

from Products.Silva.VersionedContent import VersionedContent

from silva.core import conf as silvaconf
from silva.app.news.AgendaItem import AgendaItemContent
from silva.app.news.AgendaItem import AgendaItemContentVersion
from silva.app.news.AgendaItem import AgendaItemFields
from silva.core.contentlayout.interfaces import PageFields
from zeam.form import silva as silvaforms

from ..page import PageContentVersion
from .interfaces import IAgendaPage, IAgendaPageVersion


class AgendaPageVersion(AgendaItemContentVersion, PageContentVersion):
    """An agenda page version
    """
    security = ClassSecurityInfo()
    meta_type = 'Silva Agenda Page Version'
    grok.implements(IAgendaPageVersion)


InitializeClass(AgendaPageVersion)


class AgendaPage(AgendaItemContent, VersionedContent):
    """A page that can behave as an agenda item
    """
    security = ClassSecurityInfo()
    grok.implements(IAgendaPage)
    meta_type = 'Silva Agenda Page'
    silvaconf.icon('news/agenda.png')
    silvaconf.version_class(AgendaPageVersion)
    silvaconf.priority(-8)


InitializeClass(AgendaPage)


class AgendaPageAddForm(silvaforms.SMIAddForm):
    """Add form for an agenda page.
    """
    grok.context(IAgendaPage)
    grok.name(u'Silva Agenda Page')

    fields = silvaforms.Fields(PageFields, AgendaItemFields)



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from silva.core import conf as silvaconf
from silva.core.xml import NS_SILVA_URI, handlers
from silva.app.news.silvaxml import helpers
from silva.app.page.silvaxml import NS_PAGE_URI
from silva.core.contentlayout.silvaxml import NS_LAYOUT_URI
from silva.core.contentlayout.silvaxml.xmlimport import DesignHandler,\
    BlockHandler
from silva.app.page.news.blocks import NewsInfoBlock, AgendaInfoBlock


silvaconf.namespace(NS_PAGE_URI)


class PageHandler(handlers.SilvaHandler):
    silvaconf.name('page')

    def getOverrides(self):
        return {(NS_SILVA_URI, 'content'): PageVersionHandler}

    def _createContent(self, identifier):
        factory = self.parent().manage_addProduct['silva.app.page']
        factory.manage_addPage(identifier, '', no_default_version=True)

    def startElementNS(self, name, qname, attrs):
        if name == (NS_PAGE_URI, 'page'):
            self.createContent(attrs)

    def endElementNS(self, name, qname):
        if name == (NS_PAGE_URI, 'page'):
            self.notifyImport()


class PageVersionHandler(handlers.SilvaVersionHandler):

    def getOverrides(self):
        return {(NS_LAYOUT_URI, 'design'): DesignHandler}

    def _createVersion(self, identifier):
        factory = self.parent().manage_addProduct['silva.app.page']
        factory.manage_addPageVersion(identifier, '')

    def startElementNS(self, name, qname, attrs):
        if (NS_SILVA_URI, 'content') == name:
            self.createVersion(attrs)

    def endElementNS(self, name, qname):
        if (NS_SILVA_URI, 'content') == name:
            self.updateVersionCount()
            self.storeMetadata()
            self.storeWorkflow()


class NewsPageHandler(handlers.SilvaHandler):
    silvaconf.name('news_page')

    def getOverrides(self):
        return {(NS_SILVA_URI, 'content'): NewsPageVersionHandler}

    def _createContent(self, identifier):
        factory = self.parent().manage_addProduct['silva.app.page']
        factory.manage_addNewsPage(identifier, '', no_default_version=True)

    def startElementNS(self, name, qname, attrs):
        if name == (NS_PAGE_URI, 'news_page'):
            self.createContent(attrs)

    def endElementNS(self, name, qname):
        if name == (NS_PAGE_URI, 'news_page'):
            self.notifyImport()


class NewsPageVersionHandler(handlers.SilvaVersionHandler):

    def getOverrides(self):
        return {(NS_LAYOUT_URI, 'design'): DesignHandler}

    def _createVersion(self, identifier):
        factory = self.parent().manage_addProduct['silva.app.page']
        factory.manage_addNewsPageVersion(identifier, '')

    def startElementNS(self, name, qname, attrs):
        if name == (NS_SILVA_URI, 'content'):
            version = self.createVersion(attrs)

            helpers.set_as_list(version, 'target_audiences', attrs)
            helpers.set_as_list(version, 'subjects', attrs)
            helpers.set_as_naive_datetime(version, 'display_datetime', attrs)

    def endElementNS(self, name, qname):
        if name == (NS_SILVA_URI, 'content'):
            self.updateVersionCount()
            self.storeMetadata()
            self.storeWorkflow()


class AgendaPageHandler(handlers.SilvaHandler):
    silvaconf.name('agenda_page')

    def getOverrides(self):
        return {(NS_SILVA_URI, 'content'): AgendaPageVersionHandler}

    def _createContent(self, identifier):
        factory = self.parent().manage_addProduct['silva.app.page']
        factory.manage_addAgendaPage(identifier, '', no_default_version=True)

    def startElementNS(self, name, qname, attrs):
        if name == (NS_PAGE_URI, 'agenda_page'):
            self.createContent(attrs)

    def endElementNS(self, name, qname):
        if name == (NS_PAGE_URI, 'agenda_page'):
            self.notifyImport()


class AgendaPageVersionHandler(handlers.SilvaVersionHandler):

    def getOverrides(self):
        return {(NS_LAYOUT_URI, 'design'): DesignHandler}

    def _createVersion(self, identifier):
        factory = self.parent().manage_addProduct['silva.app.page']
        factory.manage_addAgendaPageVersion(identifier, '')

    def startElementNS(self, name, qname, attrs):
        if name == (NS_SILVA_URI, 'content'):
            version = self.createVersion(attrs)

            helpers.set_as_list(version, 'target_audiences', attrs)
            helpers.set_as_list(version, 'subjects', attrs)
            helpers.set_as_naive_datetime(version, 'display_datetime', attrs)
            self.occurrences = []

    def endElementNS(self, name, qname):
        if name == (NS_SILVA_URI, 'content'):
            self.result().set_occurrences(self.occurrences)
            self.updateVersionCount()
            self.storeMetadata()
            self.storeWorkflow()


class NewsInfoBlockHandler(BlockHandler):
    silvaconf.name('newsinfoblock')

    def startElementNS(self, name, qname, attrs):
        if name == (NS_PAGE_URI, 'newsinfoblock'):
            block = NewsInfoBlock()
            self._block = block

    def endElementNS(self, name, qname):
        if name == (NS_PAGE_URI, 'newsinfoblock'):
            self.add_block(self._block)
            self._block = None


class AgendaInfoBlockHandler(BlockHandler):
    silvaconf.name('agendainfoblock')

    def startElementNS(self, name, qname, attrs):
        if name == (NS_PAGE_URI, 'agendainfoblock'):
            block = AgendaInfoBlock()
            self._block = block

    def endElementNS(self, name, qname):
        if name == (NS_PAGE_URI, 'agendainfoblock'):
            self.add_block(self._block)
            self._block = None




# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from five import grok
from zope.interface import Interface

from Products.Silva.silvaxml.xmlimport import NS_SILVA_URI

from silva.core.xml import producers
from silva.app.page.silvaxml import NS_PAGE_URI
from silva.core.contentlayout.silvaxml.xmlexport import BasePageProducer
from silva.app.page.page import Page, PageVersion
from silva.app.page.news.news import NewsPage
from silva.app.page.news.news import NewsPageVersion
from silva.app.news.silvaxml.xmlexport import NewsItemVersionProducer
from silva.app.news.silvaxml.xmlexport import AgendaItemVersionProducer
from silva.app.page.news.agenda import AgendaPage, AgendaPageVersion
from silva.app.page.news.blocks import AgendaInfoBlock, NewsInfoBlock


class PageProducer(producers.SilvaVersionedContentProducer):
    """Export a Silva Page object to XML."""
    grok.adapts(Page, Interface)

    def sax(self):
        self.startElementNS(NS_PAGE_URI, 'page', {'id': self.context.id})
        self.sax_workflow()
        self.sax_versions()
        self.endElementNS(NS_PAGE_URI, 'page')


class PageVersionProducer(BasePageProducer):
    """Export a version of a Silva Page object to XML."""
    grok.adapts(PageVersion, Interface)

    def sax(self):
        self.startElementNS(NS_SILVA_URI, 'content',
                            {'version_id': self.context.id})
        self.sax_metadata()
        self.sax_design()
        self.endElementNS(NS_SILVA_URI, 'content')


class NewsPageProducer(producers.SilvaVersionedContentProducer):
    """Export a News item Page object to XML."""
    grok.adapts(NewsPage, Interface)

    def sax(self):
        self.startElementNS(NS_PAGE_URI, 'news_page', {'id': self.context.id})
        self.sax_workflow()
        self.sax_versions()
        self.endElementNS(NS_PAGE_URI, 'news_page')


class NewsPageVersionProducer(NewsItemVersionProducer, BasePageProducer):
    """ Export news item page version to XML
    """
    grok.adapts(NewsPageVersion, Interface)

    def sax_content(self):
        self.sax_design()


class AgendaPageProducer(producers.SilvaVersionedContentProducer):
    """ Export Agenda Page to XML
    """
    grok.adapts(AgendaPage, Interface)

    def sax(self):
        self.startElementNS(NS_PAGE_URI, 'agenda_page', {'id': self.context.id})
        self.sax_workflow()
        self.sax_versions()
        self.endElementNS(NS_PAGE_URI, 'agenda_page')


class AgendaPageVersionProducer(AgendaItemVersionProducer, BasePageProducer):
    """ Export Agenda Page Version to XML
    """
    grok.adapts(AgendaPageVersion, Interface)

    def sax_content(self):
        self.sax_design()


class NewsInfoBlockProducer(producers.SilvaProducer):
    """ Export AgendaInfoBlock to XML
    """
    grok.adapts(NewsInfoBlock, Interface)

    def sax(self, parent=None):
        self.startElementNS(NS_PAGE_URI, 'newsinfoblock')
        self.endElementNS(NS_PAGE_URI, 'newsinfoblock')


class AgendaInfoBlockProducer(producers.SilvaProducer):
    """ Export AgendaInfoBlock to XML
    """
    grok.adapts(AgendaInfoBlock, Interface)

    def sax(self, parent=None):
        self.startElementNS(NS_PAGE_URI, 'agendainfoblock')
        self.endElementNS(NS_PAGE_URI, 'agendainfoblock')




# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt


import unittest
from zope.component import queryUtility
from zope.interface.verify import verifyObject

from silva.app.page.interfaces import IPage, IPageVersion
from silva.app.page.testing import FunctionalLayer
from silva.core.contentlayout.interfaces import IContentLayoutService
from silva.core.interfaces import IPublicationWorkflow


class PageTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('editor')

    def search(self, path):
        return map(lambda b: (b.getPath(), b.publication_status),
                   self.root.service_catalog(path=path))

    def test_page(self):
        factory = self.root.manage_addProduct['silva.app.page']
        factory.manage_addPage('page', 'Page')

        page = self.root._getOb('page', None)
        self.assertNotEqual(page, None)
        self.assertTrue(verifyObject(IPage, page))
        self.assertTrue(verifyObject(IPageVersion, page.get_editable()))
        self.assertEqual(page.get_viewable(), None)

    def test_page_catalog(self):
        factory = self.root.manage_addProduct['silva.app.page']
        factory.manage_addPage('page', 'Page')

        page = self.root._getOb('page')
        self.assertItemsEqual(
            self.search('/root/page'),
            [('/root/page', 'unapproved'),
             ('/root/page/0', 'unapproved')])

        IPublicationWorkflow(page).publish()
        self.assertItemsEqual(
            self.search('/root/page'),
            [('/root/page', 'public'),
             ('/root/page/0', 'public')])

    def test_installation(self):
        # Service must be installed by default by the extension
        service = queryUtility(IContentLayoutService)
        self.assertNotEqual(service, None)
        self.assertTrue(verifyObject(IContentLayoutService, service))


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(PageTestCase))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from Persistence import Persistent

from five import grok
from zope.interface import Interface
from zope.component import queryUtility, getUtility

from silva.core import conf as silvaconf
from silva.core.conf.installer import DefaultInstaller
from silva.core.contentlayout.interfaces import IContentLayoutService
from silva.core.contentlayout.interfaces import IDesignLookup
from silva.core.editor.interfaces import ICKEditorService
from silva.core.interfaces import IContainerPolicy

silvaconf.extension_name("silva.app.page")
silvaconf.extension_title(u"Silva Page")
silvaconf.extension_depends(
    ["Silva",
     "silva.core.contentlayout",
     "silva.app.news"])


class PagePolicy(Persistent):
    grok.implements(IContainerPolicy)

    def createDefaultDocument(self, container, title):
        factory = container.manage_addProduct['silva.app.page']
        factory.manage_addPage('index', title)
        page = container._getOb('index')
        service = getUtility(IDesignLookup)
        page.get_editable().set_design(service.default_design(page))


class PageInstaller(DefaultInstaller):
    """Installer for Page and Page Asset extension.
    """
    not_globally_addables = ['Silva News Page', 'Silva Agenda Page']

    def install_custom(self, root):
        if queryUtility(IContentLayoutService) is None:
            factory = root.manage_addProduct['silva.core.contentlayout']
            factory.manage_addContentLayoutService()
        if queryUtility(ICKEditorService) is None:
            factory = root.manage_addProduct['silva.core.editor']
            factory.manage_addCKEditorService()

        declare = queryUtility(ICKEditorService).declare_configuration
        declare('Silva Page')
        declare('Silva News Page', ['Silva Page'])
        declare('Silva Agenda Page', ['Silva News Page', 'Silva Page'])

        root.service_containerpolicy.register('Silva Page', PagePolicy, 0)


class IExtension(Interface):
    """Marker interface for our extension.
    """


install = PageInstaller("silva.app.page", IExtension)



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from Acquisition import aq_chain
from zope.interface.verify import verifyObject

from Products.Silva.tests.test_xml_import import SilvaXMLTestCase
from silva.app.page.news.interfaces import IAgendaPage, IAgendaPageVersion
from silva.app.page.news.interfaces import INewsPage, INewsPageVersion
from silva.app.page.news.blocks import NewsInfoBlock, AgendaInfoBlock
from silva.app.page.interfaces import IPage, IPageVersion
from silva.core.contentlayout.interfaces import IBlockManager
from silva.core.contentlayout.interfaces import IPageModel, IPageModelVersion

from ..testing import FunctionalLayer


class PageXMLImportTestCase(SilvaXMLTestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('editor')

    def test_import_page(self):
        """Import a simple page.
        """
        importer = self.assertImportFile(
            "test_import_page.silvaxml",
            ['/root/base',
             '/root/base/page'])
        self.assertEqual(importer.getProblems(), [])
        page = self.root.base._getOb('page')
        self.assertTrue(verifyObject(IPage, page))
        page_version = page.get_editable()
        self.assertTrue(verifyObject(IPageVersion, page_version))
        design = page_version.get_design()
        self.assertTrue(design)

    def test_import_with_page_model(self):
        """Import a page with a page model at the same time.
        """
        importer = self.assertImportFile(
            "test_import_page_with_page_model.silvaxml",
            ['/root/base/page',
             '/root/base/model',
             '/root/base'])
        self.assertEqual(importer.getProblems(), [])

        page = self.root.base._getOb('page')
        self.assertTrue(verifyObject(IPage, page))
        page_version = page.get_editable()
        self.assertTrue(verifyObject(IPageVersion, page_version))
        model = self.root.base._getOb('model')
        self.assertTrue(verifyObject(IPageModel, model))

        page_model = page_version.get_design()
        model_version = model.get_viewable()
        self.assertTrue(verifyObject(IPageModelVersion, page_model))
        self.assertTrue(verifyObject(IPageModelVersion, model_version))
        self.assertEqual(model_version, page_model)
        self.assertEqual(aq_chain(model_version), aq_chain(page_model))

    def test_import_with_page_model_ignore_top_level(self):
        """Import a page with a page model while ignoring the top
        level container. In that case, the page should still use the
        imported page model.
        """
        importer = self.assertImportFile(
            "test_import_page_with_page_model.silvaxml",
            ['/root/page',
             '/root/model'], ignore_top_level=True)
        self.assertEqual(importer.getProblems(), [])

        page = self.root._getOb('page')
        self.assertTrue(verifyObject(IPage, page))
        page_version = page.get_editable()
        self.assertTrue(verifyObject(IPageVersion, page_version))
        model = self.root._getOb('model')
        self.assertTrue(verifyObject(IPageModel, model))

        page_model = page_version.get_design()
        model_version = model.get_viewable()
        self.assertTrue(verifyObject(IPageModelVersion, page_model))
        self.assertTrue(verifyObject(IPageModelVersion, model_version))
        self.assertEqual(model_version, page_model)
        self.assertEqual(aq_chain(model_version), aq_chain(page_model))

    def test_import_page_existing_rename_with_external_page_model(self):
        """Import a page when an item with the same identifier
        already exists. In addition to this, the page uses a page
        model that is not included inside the import.
        """
        factory = self.root.manage_addProduct['Silva']
        factory.manage_addMockupVersionedContent('page', 'Existing page')
        factory = self.root.manage_addProduct['silva.core.contentlayout']
        factory.manage_addPageModel('model', 'Existing model')

        importer = self.assertImportFile(
            "test_import_page_with_external_page_model.silvaxml",
            ['/root/import_of_page'], ignore_top_level=True)
        self.assertEqual(importer.getProblems(), [])

    def test_import_news_page(self):
        """Import a news page.
        """
        importer = self.assertImportFile(
            "test_import_news_page.silvaxml",
            ['/root/news/index',
             '/root/news/newspage',
             '/root/news/filter',
             '/root/news'])
        self.assertEqual(importer.getProblems(), [])

        base = self.root._getOb('news')
        news_page = base._getOb('newspage')
        self.assertTrue(verifyObject(INewsPage, news_page))
        version = news_page.get_editable()
        self.assertTrue(verifyObject(INewsPageVersion, version))
        self.assertIsNot(version.get_design(), None)

        manager = IBlockManager(version)
        slot = manager.get_slot('one')
        _, block = slot[0]
        self.assertIsInstance(block, NewsInfoBlock)

    def test_import_agenda_page(self):
        """Import an agenda page.
        """
        importer = self.assertImportFile(
            "test_import_agenda_page.silvaxml",
            ['/root/news/index',
             '/root/news/agendapage',
             '/root/news/filter',
             '/root/news'])
        self.assertEqual(importer.getProblems(), [])

        base = self.root._getOb('news')
        agenda_page = base._getOb('agendapage')
        self.assertTrue(verifyObject(IAgendaPage, agenda_page))
        version = agenda_page.get_editable()
        self.assertTrue(verifyObject(IAgendaPageVersion, version))
        design = version.get_design()
        self.assertIsNot(design, None)

        manager = IBlockManager(version)
        slot = manager.get_slot('one')
        _, block = slot[0]
        self.assertIsInstance(block, AgendaInfoBlock)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(PageXMLImportTestCase))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt


from silva.app.news.interfaces import INewsItemContent, INewsItemContentVersion
from silva.app.news.interfaces import IAgendaItemContent
from silva.app.news.interfaces import IAgendaItemContentVersion
from silva.app.page.interfaces import IPageContent, IPageContentVersion


class INewsPageVersion(INewsItemContentVersion, IPageContentVersion):
    """A news page version
    """


class INewsPage(INewsItemContent, IPageContent):
    """A news page.
    """


class IAgendaPageVersion(IAgendaItemContentVersion, IPageContentVersion):
    """A agenda page version
    """


class IAgendaPage(IAgendaItemContent, IPageContent):
    """An agenda page
    """



# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt


from silva.core.contentlayout.testing import SilvaContentLayoutLayer
import silva.app.page
import transaction


class SilvaPageLayer(SilvaContentLayoutLayer):
    default_packages = SilvaContentLayoutLayer.default_packages + [
        'silva.core.editor',
        'silva.app.news',
        'silva.app.page',
        ]

    def _install_application(self, app):
        super(SilvaPageLayer, self)._install_application(app)
        app.root.service_extensions.install('silva.core.contentlayout')
        app.root.service_extensions.install('silva.app.news')
        app.root.service_extensions.install('silva.app.page')
        transaction.commit()

FunctionalLayer = SilvaPageLayer(silva.app.page)



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import localdatetime

from datetime import datetime
from five import grok
from grokcore.chameleon.components import ChameleonPageTemplate
from zope.cachedescriptors.property import Lazy
from zope.event import notify
from zope.lifecycleevent import ObjectModifiedEvent
from zope.publisher.interfaces.http import IHTTPRequest
from zope.traversing.browser import absoluteURL

from silva.app.news.datetimeutils import RRuleData
from silva.core.contentlayout.blocks import Block, BlockController
from silva.translations import translate as _
from silva.ui.rest import UIREST

from .interfaces import INewsPageVersion, IAgendaPageVersion


class NewsInfoBlock(Block):
    grok.context(INewsPageVersion)
    grok.name('news-information')
    grok.title(_(u'News information'))
    grok.order(50)


class AgendaInfoBlock(Block):
    grok.context(IAgendaPageVersion)
    grok.name('agenda-information')
    grok.title(_(u'Agenda information'))
    grok.order(50)


class NewsInfoBlockController(BlockController):
    grok.adapts(NewsInfoBlock, INewsPageVersion, IHTTPRequest)

    template = ChameleonPageTemplate(filename="templates/newsinfo.cpt")

    def __init__(self, block, version, request):
        self.version = version
        self.request = request

    @Lazy
    def url(self):
        return absoluteURL(self.version.get_content(), self.request)

    @Lazy
    def month_names(self):
        return localdatetime.get_month_names(self.request)

    def format_date(self, date, with_hours=True):
        if not isinstance(date, datetime):
            date = date.asdatetime()
        formatted = u'%s.%s.%s' % (
            date.day, self.month_names[date.month-1], date.year)
        if with_hours:
            formatted += u', %s:%s' % (
                '%02d' % date.hour, '%02d' % date.minute)
        return formatted

    @Lazy
    def publication_date(self):
        date = self.version.get_display_datetime()
        if date:
            return self.format_date(date)
        return u''

    def default_namespace(self):
        return {}

    def namespace(self):
        return {'version': self.version,
                'request': self.request,
                'view': self}

    def render(self, view=None):
        if view is None or view.final:
            return self.template.render(self)
        return _(u"This component is not available in this context.")


class AgendaInfoBlockController(NewsInfoBlockController):
    grok.adapts(AgendaInfoBlock, IAgendaPageVersion, IHTTPRequest)

    template = ChameleonPageTemplate(filename="templates/agendainfo.cpt")

    def occurrences(self):
        for occurrence in self.version.get_occurrences():
            timezone = occurrence.get_timezone()
            with_hours = not occurrence.is_all_day()

            start = occurrence.get_start_datetime(timezone)
            end = occurrence.get_end_datetime(timezone)
            end_recurrence = occurrence.get_end_recurrence_datetime()

            information = {
                'start': self.format_date(start, with_hours),
                'end': self.format_date(end, with_hours),
                'location': occurrence.get_location(),
                'recurrence_until': None}

            if end_recurrence:
                information.update({
                    'recurrence_until': self.format_date(
                        end_recurrence, with_hours),
                    'recurrence': RRuleData(
                        occurrence.get_recurrence()).get('FREQ')})

            yield information


class AddBlockREST(UIREST):
    grok.baseclass()
    grok.name('add')
    grok.require('silva.ChangeSilvaContent')

    message = _(u'Component created.')
    autoclose = 4000

    def __init__(self, context, request, configuration, restriction):
        super(AddBlockREST, self).__init__(context, request)
        self.configuration = configuration
        self.restriction = restriction

    def _create_block(self):
        raise NotImplementedError

    def GET(self):
        adding = self.__parent__
        adding.add(self._create_block())
        notify(ObjectModifiedEvent(self.context))

        return self.json_response({
                'content' : {
                    'extra': {
                        'block_id': adding.block_id,
                        'block_data': adding.block_controller.render(),
                        'block_editable': False},
                    'success': True},
                "notifications": [{
                        "category": "",
                        "message": self.message,
                        "autoclose": self.autoclose}],
                })


class AddNewsBlockREST(AddBlockREST):
    grok.adapts(NewsInfoBlock, INewsPageVersion)

    message = _(u'News information component added.')

    def _create_block(self):
        return NewsInfoBlock()


class AddAgendaBlockREST(AddBlockREST):
    grok.adapts(AgendaInfoBlock, IAgendaPageVersion)

    message = _(u'Agenda information component added.')

    def _create_block(self):
        return AgendaInfoBlock()



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt
# this is a package

from silva.core.xml import registerNamespace

NS_PAGE_URI = 'http://infrae.com/namespace/silva-app-page'
registerNamespace('silva-app-page', NS_PAGE_URI)




# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from five import grok

from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass

from Products.Silva.VersionedContent import VersionedContent

from silva.core import conf as silvaconf
from silva.app.news.NewsItem import NewsItemContent, NewsItemFields
from silva.app.news.NewsItem import NewsItemContentVersion
from silva.core.contentlayout.interfaces import PageFields
from zeam.form import silva as silvaforms

from ..page import PageContentVersion
from .interfaces import INewsPage, INewsPageVersion


class NewsPageVersion(NewsItemContentVersion, PageContentVersion):
    """A news page version
    """
    security = ClassSecurityInfo()
    meta_type = 'Silva News Page Version'
    grok.implements(INewsPageVersion)


InitializeClass(NewsPageVersion)


class NewsPage(NewsItemContent, VersionedContent):
    """A page that can behave like a news item
    """
    security = ClassSecurityInfo()
    grok.implements(INewsPage)
    meta_type = 'Silva News Page'
    silvaconf.icon('news/news.png')
    silvaconf.version_class(NewsPageVersion)
    silvaconf.priority(-9)


InitializeClass(NewsPage)


class NewsPageAddForm(silvaforms.SMIAddForm):
    """Add form for an agenda page.
    """
    grok.context(INewsPage)
    grok.name(u'Silva News Page')

    fields = silvaforms.Fields(PageFields, NewsItemFields)



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt
from setuptools import setup, find_packages
import os

version = '3.0.4'

tests_require = [
    'Products.Silva [test]',
    'silva.app.news',
    ]

setup(name='silva.app.page',
      version=version,
      description="Silva page content types for Silva CMS",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      classifiers=[
          "Environment :: Web Environment",
          "Programming Language :: Python",
          "Topic :: Software Development :: Libraries :: Python Modules",
          "Framework :: Zope2",
        ],
      keywords='silva contentlayout zope cms',
      author='Infrae',
      author_email='info@infrae.com',
      url='https://github.com/silvacms/silva.app.page',
      license='GPL',
      package_dir={'': 'src'},
      packages=find_packages('src'),
      namespace_packages=['silva', 'silva.app'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'five.grok',
          'setuptools',
          'silva.core.conf',
          'silva.core.contentlayout',
          'silva.core.interfaces',
          'silva.core.views',
          'silva.core.xml',
          'silva.translations',
          'silva.ui',
          'zeam.form.silva',
          'zope.component',
          'zope.event',
          'zope.i18n',
          'zope.interface',
          'zope.lifecycleevent',
          ],
      tests_require = tests_require,
      extras_require = {'test': tests_require},
      )



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from five import grok
from zope.component import getMultiAdapter
from zope.i18n import translate
from zope.publisher.browser import TestRequest

from AccessControl import ClassSecurityInfo
from AccessControl.security import checkPermission
from App.class_init import InitializeClass

from Products.Silva.VersionedContent import VersionedContent
from Products.Silva.Version import Version

from silva.core import conf as silvaconf
from silva.core.contentlayout.interfaces import PageFields
from silva.core.contentlayout.interfaces import IBoundBlockManager
from silva.core.contentlayout.designs.design import DesignAccessors

from silva.core.interfaces.adapters import IIndexEntries
from silva.core.smi.content import IEditScreen
from silva.core.smi.settings import Settings
from silva.core.views import views as silvaviews
from silva.translations import translate as _
from silva.ui.interfaces import IJSView
from silva.ui.rest.base import Screen, PageREST
from zeam.form import silva as silvaforms

from .interfaces import IPage, IPageVersion, IPageContent
from .interfaces import IPageContentVersion


class PageContentVersion(Version, DesignAccessors):
    grok.baseclass()
    grok.implements(IPageContentVersion)

    def fulltext(self):
        fulltext = [self.get_title()]
        manager = getMultiAdapter(
            (self, TestRequest()), IBoundBlockManager)

        def collect(block_id, controller):
            fulltext.extend(controller.fulltext())

        manager.visit(collect)
        return fulltext


class PageVersion(PageContentVersion):
    """A version of a Silva Page (i.e. a web page)
    """
    security = ClassSecurityInfo()
    meta_type = 'Silva Page Version'
    grok.implements(IPageVersion)


InitializeClass(PageVersion)


class Page(VersionedContent):
    """ A Silva Page represents a web page, supporting advanced
        inline editing and content layout.
    """
    security = ClassSecurityInfo()
    grok.implements(IPage)
    meta_type = 'Silva Page'
    silvaconf.icon('page.png')
    silvaconf.version_class(PageVersion)
    silvaconf.priority(-10)


InitializeClass(Page)


class PageAddForm(silvaforms.SMIAddForm):
    """Add form for a page asset"""

    grok.context(IPage)
    grok.name(u'Silva Page')

    fields = PageFields


class PageEdit(PageREST):
    grok.adapts(Screen, IPageContent)
    grok.name('content')
    grok.implements(IEditScreen)
    grok.require('silva.ReadSilvaContent')

    def payload(self):
        if checkPermission('silva.ChangeSilvaContent', self.context):
            version = self.context.get_editable()
            if version is not None:
                view = getMultiAdapter(
                    (version, self.request), IJSView, name='content-layout')
                return view(self, identifier=version.getId())

        view = getMultiAdapter(
            (self.context, self.request), IJSView, name='content-preview')
        return view(self)


class PageDesignForm(silvaforms.SMISubEditForm):
    grok.context(IPageContent)
    grok.view(Settings)
    grok.order(10)

    label = _(u"Page template")
    fields = PageFields.omit('id', 'title')


class PageView(silvaviews.View):
    grok.context(IPageContent)

    def render(self):
        if self.content is not None:
            design = self.content.get_design()
            if design is not None:
                render = design(self.content, self.request, [self.content])
                if render is not None:
                    return render()
        msg = _('Sorry, this ${meta_type} is not viewable.',
                mapping={'meta_type': self.context.meta_type})
        return '<p>%s</p>' % translate(msg, context=self.request)


#Indexes
class PageIndexEntries(grok.Adapter):
    grok.implements(IIndexEntries)
    grok.context(IPageContent)

    def get_title(self):
        return self.context.get_title()

    def get_entries(self):
        version = self.context.get_viewable()
        if version is None:
            return []

        indexes = []
        manager = getMultiAdapter(
            (version, TestRequest()), IBoundBlockManager)

        def collect(block_id, controller):
            indexes.extend(controller.indexes())

        manager.visit(collect)

        return indexes



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt


import unittest
from zope.interface.verify import verifyObject

from silva.core.interfaces import IAddableContents
from silva.core.interfaces import IPublicationWorkflow
from silva.app.page.news.interfaces import INewsPage, INewsPageVersion
from silva.app.page.news.interfaces import IAgendaPage, IAgendaPageVersion
from silva.app.page.testing import FunctionalLayer


class NewsPageTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('editor')
        factory = self.root.manage_addProduct['silva.app.news']
        factory.manage_addNewsPublication('news', 'News')
        # Show agenda items in the filter.
        self.root.news.filter.set_show_agenda_items(True)

    def search(self, path):
        return map(lambda b: (b.getPath(), b.publication_status),
                   self.root.service_catalog(path=path))

    def test_news_addables(self):
        """News page and Agenda page should be addable in a news
        publication by default.
        """
        root_addables = IAddableContents(self.root).get_container_addables()
        self.assertFalse('Silva News Page' in root_addables)
        self.assertFalse('Silva Agenda Page' in root_addables)
        # XXX Addable API need to be fixed for this.
        #root_addables = IAddableContents(self.root).get_all_addables()
        #self.assertFalse('Silva News Page' in root_addables)
        #self.assertFalse('Silva Agenda Page' in root_addables)

        news_addables = IAddableContents(self.root.news).get_container_addables()
        self.assertTrue('Silva News Page' in news_addables)
        self.assertTrue('Silva Agenda Page' in news_addables)
        news_addables = IAddableContents(self.root.news).get_all_addables()
        self.assertTrue('Silva News Page' in news_addables)
        self.assertTrue('Silva Agenda Page' in news_addables)


    def test_news_page(self):
        factory = self.root.news.manage_addProduct['silva.app.page']
        factory.manage_addNewsPage('item', 'Page item')

        page = self.root.news._getOb('item')
        self.assertTrue(verifyObject(INewsPage, page))
        self.assertTrue(verifyObject(INewsPageVersion, page.get_editable()))
        self.assertEqual(page.get_viewable(), None)

        IPublicationWorkflow(page).publish()
        self.assertTrue(verifyObject(INewsPageVersion, page.get_viewable()))
        self.assertEqual(page.get_editable(), None)

        # When the page is published, the news will appear in the filter.
        self.assertItemsEqual(
            map(lambda b: b.getObject(),
                self.root.news.filter.get_all_items()),
            [page.get_viewable()])

    def test_news_page_catalog(self):
        factory = self.root.news.manage_addProduct['silva.app.page']
        factory.manage_addNewsPage('item', 'Page item')

        page = self.root.news._getOb('item')
        self.assertItemsEqual(
            self.search('/root/news/item'),
            [('/root/news/item/0', 'unapproved'),
             ('/root/news/item', 'unapproved')])

        IPublicationWorkflow(page).publish()
        self.assertItemsEqual(
            self.search('/root/news/item'),
            [('/root/news/item/0', 'public'),
             ('/root/news/item', 'public')])

    def test_agenda_page(self):
        factory = self.root.news.manage_addProduct['silva.app.page']
        factory.manage_addAgendaPage('event', 'Page event')

        page = self.root.news._getOb('event')
        self.assertTrue(verifyObject(IAgendaPage, page))
        self.assertTrue(verifyObject(IAgendaPageVersion, page.get_editable()))
        self.assertEqual(page.get_viewable(), None)

        IPublicationWorkflow(page).publish()
        self.assertEqual(page.get_editable(), None)
        self.assertTrue(verifyObject(IAgendaPageVersion, page.get_viewable()))

        # When the page is published, the event will appear in the filter.
        self.assertItemsEqual(
            map(lambda b: b.getObject(),
                self.root.news.filter.get_all_items()),
            [page.get_viewable()])

    def test_agenda_page_catalog(self):
        factory = self.root.news.manage_addProduct['silva.app.page']
        factory.manage_addAgendaPage('event', 'Page event')

        page = self.root.news._getOb('event')
        self.assertItemsEqual(
            self.search('/root/news/event'),
            [('/root/news/event/0', 'unapproved'),
             ('/root/news/event', 'unapproved')])

        IPublicationWorkflow(page).publish()
        self.assertItemsEqual(
            self.search('/root/news/event'),
            [('/root/news/event/0', 'public'),
             ('/root/news/event', 'public')])

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(NewsPageTestCase))
    return suite
